<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:saxon="http://icl.com/saxon" extension-element-prefixes="saxon">

  <xsl:variable name="globalIndex" select="1" saxon:assignable="yes"/>
  <xsl:output method="xml" encoding="UTF-8"/>

  <xsl:template match="/">
    <TestRun>
      <xsl:call-template name="FailedTests"/>
      <xsl:call-template name="SuccessfulTests"/>
      <xsl:call-template name="Statistics"/>
    </TestRun>
  </xsl:template> 
 
  <xsl:template name="index">
	 <xsl:value-of select="$globalIndex"/>
	 <saxon:assign name="globalIndex" select="$globalIndex + 1" />
  </xsl:template>
  
  <xsl:template name="SuccessfulTests">
    <SuccessfulTests>
      <xsl:for-each select="/ResultsSession/ExecutedTestsDetails/Total/Project/TestSuite//TestCase[@pass > 0] |
   					       /ResultsSession/ExecutedTestsDetails/Total/Project/TestSuite//Test[@total = @pass][@pass > 0][count(TestCase) = 0]">
		
		<xsl:if test="name() = 'Test'">
			<xsl:element name="Test">
		      <xsl:attribute name="id">
			  <xsl:call-template name="index"/>
		      </xsl:attribute>
		      <Name><xsl:value-of select="@name"/></Name>
		    </xsl:element>
	    </xsl:if>
		<xsl:if test="name() = 'TestCase'">
			<xsl:element name="Test">
		      <xsl:attribute name="id">
			  <xsl:call-template name="index"/>
		      </xsl:attribute>
		      <Name><xsl:value-of select="../@name"/></Name>
		    </xsl:element>
	    </xsl:if>
      </xsl:for-each>
    </SuccessfulTests>
  </xsl:template> 
  
  <xsl:template name="FailedTests">
    <FailedTests>      
		<xsl:for-each select="/ResultsSession/ExecutedTestsDetails/Total/Project/TestSuite//Test[@fail > 0 or @err > 0][count(TestCase) = 0] |
		                      /ResultsSession/ExecutedTestsDetails/Total/Project/TestSuite//TestCase[@fail > 0 or @err > 0]">
		<!-- xsl:sort select="@name" /  -->
			<xsl:if test="name() = 'Test'">
				<xsl:element name="FailedTest">		
			      <xsl:variable name="id" select="@id" />		
			      <xsl:variable name="locRef" select="@locRef" />
			      <xsl:attribute name="id">
				    <xsl:call-template name="index"/>
			      </xsl:attribute>
			      <Name><xsl:value-of select="@name"/></Name>
			      <xsl:call-template name="FailureType"/>
			      <xsl:if test="count(/ResultsSession/Exec/ExecViols/ExecViol[@testId = $id]) > 0">
			      	<Message>
			        	<xsl:value-of select="/ResultsSession/Exec/ExecViols/ExecViol[@testId = $id]/Thr/ThrPart/@prnMsg"/>
				  	</Message>
				  	<Location>
				    	<File><xsl:value-of select="/ResultsSession/Exec/ExecViols/ExecViol[@testId = $id]/@locFile"/></File>	  
			      		<Line><xsl:value-of select="/ResultsSession/Exec/ExecViols/ExecViol[@testId = $id]/@ln"/></Line>
			      	</Location>
			      </xsl:if>
			    </xsl:element>
			</xsl:if>
		    <xsl:if test="name() = 'TestCase'">
			    <xsl:element name="FailedTest">      		
			    <xsl:variable name="id" select="@testId" />		
			    <xsl:variable name="tcId" select="@id" />
		        <xsl:variable name="locRef" select="../@locRef" />
		        <xsl:attribute name="id">
				  <xsl:call-template name="index"/>
		        </xsl:attribute>
		        <Name><xsl:value-of select="../@name"/></Name>
		        <xsl:call-template name="FailureType"/>
		        <xsl:if test="count(/ResultsSession/Exec/ExecViols/ExecViol[@testId = $id][@testCaseId = $tcId]) > 0">
				  <Message>
					<xsl:value-of select="/ResultsSession/Exec/ExecViols/ExecViol[@testId = $id][@testCaseId = $tcId]/Thr/ThrPart/@prnMsg"/>
				  </Message>
			  		<Location>
				    	<File><xsl:value-of select="/ResultsSession/Exec/ExecViols/ExecViol[@testId = $id][@testCaseId = $tcId]/@locFile"/></File>
			    		<Line><xsl:value-of select="/ResultsSession/Exec/ExecViols/ExecViol[@testId = $id][@testCaseId = $tcId]/@ln"/></Line>	  
		      		</Location>
		      	</xsl:if>
	    		</xsl:element>	
	      	</xsl:if>                     
		</xsl:for-each>
    </FailedTests>
  </xsl:template>

  <xsl:template name="FailureType">
  	<xsl:if test="@err > 0">
	  <FailureType>Error</FailureType>
    </xsl:if>      
    <xsl:if test="@fail > 0">
	  <FailureType>Assertion</FailureType>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name="Statistics">
    <Statistics>
    	<xsl:choose>
	    	<xsl:when test="/ResultsSession/ExecutedTestsDetails">
			    <Tests><xsl:value-of select="/ResultsSession/ExecutedTestsDetails/Total/@total"/></Tests>
			    <FailuresTotal><xsl:value-of select="count(/ResultsSession/Exec/ExecViols/ExecViol)"/></FailuresTotal>
			    <Errors>
			    	<xsl:value-of select="/ResultsSession/ExecutedTestsDetails/Total/@err"/>
			    	<xsl:if test="count(/ResultsSession/ExecutedTestsDetails/Total/@err) = 0">0</xsl:if>
			    </Errors>			    
			    <Failures>
			    	<xsl:value-of select="/ResultsSession/ExecutedTestsDetails/Total/@fail"/>
			    	<xsl:if test="count(/ResultsSession/ExecutedTestsDetails/Total/@fail) = 0">0</xsl:if>
		    	</Failures>
		    </xsl:when>
		    <xsl:otherwise>
			    <Tests>0</Tests>
			    <FailuresTotal>0</FailuresTotal>
			    <Errors>0</Errors>
			    <Failures>0</Failures>
		    </xsl:otherwise>
	    </xsl:choose>
    </Statistics>
  </xsl:template>

</xsl:stylesheet>


